/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.audio;

import java.util.Comparator;


/**
 * Comparator object for MP3 objects.
 * Set aSortType to any of SORT enums values before using it.
 */
public class AudioTrackComparator implements Comparator<AudioTrack> {
    public enum SORT {
        COMPARE_TITLE,
        COMPARE_FILE,
        COMPARE_SIZE,
        COMPARE_TIME,
        COMPARE_SELECTED,
        COMPARE_TRACK,
        COMPARE_ALBUM,
        COMPARE_ARTIST,
        COMPARE_GENRE,
    }


    public SORT aSortType;


    /**
     *
     */
    public AudioTrackComparator() {
        aSortType = SORT.COMPARE_FILE;
    }


    /**
     * Compare two Mp3 objects.
     *
     * @param mp3a
     * @param mp3b
     * @return
     */
    public int compare(AudioTrack mp3a, AudioTrack mp3b) {
        int tracka = 0;
        int trackb = 0;
        String s1;
        String s2;

        switch (aSortType) {
            case COMPARE_TITLE:
                return mp3a.aTitle.compareTo(mp3b.aTitle);

            case COMPARE_FILE:
                return mp3a.aFileName.compareTo(mp3b.aFileName);

            case COMPARE_SIZE:
                if (mp3a.aFileSize == mp3b.aFileSize) {
                    return mp3a.aFileName.compareTo(mp3b.aFileName);
                }
                else {
                    return mp3a.aFileSize > mp3b.aFileSize ? 1 : -1;
                }

            case COMPARE_TIME:
                if (mp3a.aTimeSize == mp3b.aTimeSize) {
                    return mp3a.aFileName.compareTo(mp3b.aFileName);
                }
                else {
                    return mp3a.aTimeSize > mp3b.aTimeSize ? 1 : -1;
                }

            case COMPARE_SELECTED:
                if (mp3a.isSelected() == mp3b.isSelected()) {
                    return mp3a.aFileName.compareTo(mp3b.aFileName);
                }
                else if (!mp3a.isSelected() && !mp3b.isSelected()) {
                    return mp3a.aFileName.compareTo(mp3b.aFileName);
                }
                else {
                    return mp3a.isSelected() ? 1 : -1;
                }

            case COMPARE_TRACK:
                try {
                    tracka = Integer.valueOf(mp3a.aTrack);
                }
                catch (NumberFormatException e) {
                }

                try {
                    trackb = Integer.valueOf(mp3b.aTrack);
                }
                catch (NumberFormatException e) {
                }

                if (tracka == trackb) {
                    return mp3a.aFileName.compareTo(mp3b.aFileName);
                }
                else {
                    return tracka > trackb ? 1 : -1;
                }

            case COMPARE_ARTIST:
                try {
                    tracka = Integer.valueOf(mp3a.aTrack);
                }
                catch (NumberFormatException e) {
                }

                try {
                    trackb = Integer.valueOf(mp3b.aTrack);
                }
                catch (NumberFormatException e) {
                }

                s1 = String.format("%s%s%04d", mp3a.aArtist, mp3a.aAlbum, tracka);
                s2 = String.format("%s%s%04d", mp3b.aArtist, mp3b.aAlbum, trackb);
                return s1.compareTo(s2);

            case COMPARE_ALBUM:
                try {
                    tracka = Integer.valueOf(mp3a.aTrack);
                }
                catch (NumberFormatException e) {
                }

                try {
                    trackb = Integer.valueOf(mp3b.aTrack);
                }
                catch (NumberFormatException e) {
                }

                s1 = String.format("%s%04d", mp3a.aAlbum, tracka);
                s2 = String.format("%s%04d", mp3b.aAlbum, trackb);
                return s1.compareTo(s2);

            case COMPARE_GENRE:
                try {
                    tracka = Integer.valueOf(mp3a.aTrack);
                }
                catch (NumberFormatException e) {
                }

                try {
                    trackb = Integer.valueOf(mp3b.aTrack);
                }
                catch (NumberFormatException e) {
                }

                s1 = String.format("%s%s%s%04d", mp3a.aGenre, mp3a.aArtist, mp3a.aAlbum, tracka);
                s2 = String.format("%s%s%s%04d", mp3b.aGenre, mp3b.aArtist, mp3b.aAlbum, trackb);
                return s1.compareTo(s2);

            default:
                return 0;
        }
    }
}
