/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.tracktab;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioGenre;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.gui.ImagePreview;
import com.googlepages.dronten.jtagger.model.GenreModel;
import com.googlepages.dronten.jtagger.model.PictureModel;
import com.googlepages.dronten.jtagger.util.IOUtils;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Pref;
import com.googlepages.dronten.jtagger.util.Util;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;


/**
 * A panel with some of the meta tags that exists for an audio track.
 */
public class TrackOptionsPanel extends JPanel implements ActionListener {
    private static final long   serialVersionUID = 1;
    private AudioTrack          aAudioTrack = null;
    private JTextField          aArtist;
    private JTextField          aAlbum;
    private JTextField          aYear;
    private JComboBox           aGenre;
    private JComboBox           aGenre2;
    private JTextField          aTrack;
    private JTextField          aTitle;
    private JTextField          aTime;
    private JTextField          aComment;
    private JTextField          aCopyright;
    private JTextField          aEncodedBy;
    private JTextField          aComposer;
    private JTextField          aBitRate;
    private JTextField          aPictureDescription;
    private PictureModel        aPictureModel;
    private JButton             aOpenFileButton;
    private JButton             aDeletePictureButton;
    private JButton             aUpdateButton;
    private JFileChooser        aFileChooser;
    private boolean             aImageSet = false;


    /**
     * Create track panel with all child components.
     */
    public TrackOptionsPanel() {
        super();

        JPanel top = new JPanel();
        JPanel bottom = new JPanel();
        top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
        bottom.setLayout(new BoxLayout(bottom, BoxLayout.Y_AXIS));
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 3, 5, 3));

        /*********************************************************************/

        top.add(ComponentFactory.createLabel("Artist:", ""));
        aArtist = ComponentFactory.createInput("", "");
        top.add(aArtist);

        top.add(ComponentFactory.createLabel("Album:", ""));
        aAlbum = ComponentFactory.createInput("", "");
        top.add(aAlbum);

        top.add(ComponentFactory.createLabel("Title:", ""));
        aTitle = ComponentFactory.createInput("", "");
        top.add(aTitle);

        top.add(ComponentFactory.createLabel("Genre ID3v1:", "ID3v 1.1 tag checkbox must be set in the setup"));
        aGenre = ComponentFactory.createCombo(new GenreModel(), 0, "", 0, 0);
        top.add(aGenre);

        top.add(ComponentFactory.createLabel("Genre ID3v2:", "ID3v 2.3 tag checkbox must be set in the setup"));
        aGenre2 = ComponentFactory.createCombo(new GenreModel(), 0, "", 0, 0);
        aGenre2.setEditable(true);
        top.add(aGenre2);

        top.add(ComponentFactory.createLabel("Year:", ""));
        aYear = ComponentFactory.createInput("", "");
        top.add(aYear);

        top.add(ComponentFactory.createLabel("Track:", ""));
        aTrack = ComponentFactory.createInput("", "");
        top.add(aTrack);

        top.add(ComponentFactory.createLabel("Comment:", ""));
        aComment = ComponentFactory.createInput("", "");
        top.add(aComment);

        top.add(ComponentFactory.createLabel("Copyright:", ""));
        aCopyright = ComponentFactory.createInput("", "<html>This tag doesn't exist in MP3 ID3v 1 tags!<br>Set year and then space and copyright message.<br>YYYY Copyright Example</html>");
        top.add(aCopyright);

        top.add(ComponentFactory.createLabel("Composer:", ""));
        aComposer = ComponentFactory.createInput("", "This tag doesn't exist in MP3 ID3v 1 tags!");
        top.add(aComposer);

        top.add(ComponentFactory.createLabel("Encoded By:", ""));
        aEncodedBy = ComponentFactory.createInput("", "This tag doesn't exist in MP3 ID3v 1 tags!");
        top.add(aEncodedBy);

        top.add(ComponentFactory.createLabel("Time - Filesize - Tagtype:", ""));
        aTime = ComponentFactory.createInput("", "");
        aTime.setEditable(false);
        top.add(aTime);

        top.add(ComponentFactory.createLabel("Bitrate - Frequency - Channels - Version:", ""));
        aBitRate = ComponentFactory.createInput("", "");
        aBitRate.setEditable(false);
        top.add(aBitRate);

        /*********************************************************************/

        aPictureDescription = ComponentFactory.createInput("", "Description of the picture");
        top.add(ComponentFactory.createLabel("Picture Label:", ""));
        top.add(aPictureDescription);
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aPictureModel = new PictureModel();
        aPictureModel.setPictureLabel(new JLabel());
        aPictureModel.getPictureLabel().setIcon(JTagger.NO_IMAGE);
        top.add(aPictureModel.getPictureLabel());
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aOpenFileButton = ComponentFactory.createButton("Load Picture...", "Load and set the first picture in the audio track", this);
        aOpenFileButton.setEnabled(false);
        top.add(aOpenFileButton);
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aDeletePictureButton = ComponentFactory.createButton("Remove Picture", "Delete first picture in the audio track", this);
        aDeletePictureButton.setEnabled(false);
        top.add(aDeletePictureButton);
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        /*********************************************************************/

        aUpdateButton = ComponentFactory.createButton("Write Tags", "Save all changes to audio track", this);
        aUpdateButton.setEnabled(false);
        bottom.add(aUpdateButton);

        /*********************************************************************/

        add(top, BorderLayout.NORTH);
        add(bottom, BorderLayout.SOUTH);

        setMinimumSize(new Dimension(150, Short.MAX_VALUE));
        setPreferredSize(new Dimension(250, Short.MAX_VALUE));

        /*********************************************************************/

        String path = Pref.getPref(Constants.PREF_LASTDDIR_PICTURE, "");

        aFileChooser = new JFileChooser();
        aFileChooser.setCurrentDirectory(new File(path));
        aFileChooser.setFileFilter(new FileFilter() {

            /**
             *
             * @param f
             * @return
             */
            @Override
            public boolean accept(File f) {
                return f.getName().endsWith(".jpg")||f.getName().endsWith(".gif")||f.getName().endsWith(".png")||f.isDirectory();
            }


            /**
             *
             * @return
             */
            @Override
            public String getDescription() {
                return "JPG & GIF & PNG Images";
            }

        });

        aFileChooser.setAccessory(new ImagePreview(aFileChooser));
    }


    /**
     * All button pushes goes here.
     *
     * @param actionEvent Event message
     */
    @Override public void actionPerformed(ActionEvent actionEvent) {
        if (actionEvent.getSource() == aUpdateButton) {
            writeTags();
        }
        else if (actionEvent.getSource() == aOpenFileButton) {
            int returnVal = aFileChooser.showOpenDialog(TrackOptionsPanel.this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = aFileChooser.getSelectedFile();

                if (file != null){
                    try {
                        ImageIcon icon = new ImageIcon(file.toURI().toURL());
                        icon = new ImageIcon(icon.getImage().getScaledInstance(110, 110, Image.SCALE_DEFAULT));
                        aPictureModel.getPictureLabel().setIcon(icon);
                        aPictureModel.setPictureData(IOUtils.toByteArray(file));
                        aPictureModel.setMimeType(IOUtils.getMimeTypeFromFile(file.getName()));
                        aImageSet = true;
                        aDeletePictureButton.setEnabled(true);
                        Log.get().addTime(1,"Loaded image: " + file.getName());
                    }
                    catch (Exception e) {
                        JTagger.get().setMessage(e.getMessage(), true);
                    }
                }
            }

            Pref.setPref(Constants.PREF_LASTDDIR_PICTURE, aFileChooser.getCurrentDirectory().getAbsolutePath());
        }
        else if (actionEvent.getSource() == aDeletePictureButton) {
            aAudioTrack.aPictures = null;
            aAudioTrack.aClearedOfPicture = true;
            aPictureModel.getPictureLabel().setIcon(JTagger.NO_IMAGE);
            aPictureDescription.setText("");
            aPictureModel.setPictureData(null);
       }


    }


    /**
     * Clear all fields and select default genre "Other".
     */
    private void clear() {
        aArtist.setText("");
        aAlbum.setText("");
        aTitle.setText("");
        aYear.setText("");
        aGenre.getModel().setSelectedItem(AudioGenre.DEFAULT_STRING);
        aGenre2.getModel().setSelectedItem(AudioGenre.DEFAULT_STRING);
        aTrack.setText("");
        aBitRate.setText("");
        aTime.setText("");
        aComment.setText("");
        aCopyright.setText("");
        aComposer.setText("");
        aEncodedBy.setText("");
        aPictureDescription.setText("");
        aPictureModel.getPictureLabel().setIcon(JTagger.NO_IMAGE);
        aPictureModel.setPictureData(null);
    }


    /**
     * Set mp3 track and display all track tags.
     *
     * @param track
     */
    public void setTrack(AudioTrack track) {
        aAudioTrack = track;
        String tagType;

        try {
            if (track != null) {
                if (track.aClearedOfPicture) {
                    track.read();
                }

                if (aAudioTrack.aType == AudioTrack.TrackType.MP3_ID3V1) {
                    tagType= "ID3v1";
                }
                else if (aAudioTrack.aType == AudioTrack.TrackType.MP3_ID3V2) {
                    tagType= "ID3v2";
                }
                else if (aAudioTrack.aType == AudioTrack.TrackType.MP3_ID3V) {
                    tagType= "ID3v1 & ID3v2";
                }
                else {
                    tagType= "No Tags";
                }

                aImageSet = false;
                aAudioTrack = track;
                aArtist.setText(aAudioTrack.aArtist);
                aAlbum.setText(aAudioTrack.aAlbum);
                aTitle.setText(aAudioTrack.aTitle);
                aTime.setText(String.format("%s - %d b (%.1f MB) - %s", aAudioTrack.aTime, aAudioTrack.aFileSize, ((double) aAudioTrack.aFileSize) / 1048576.0, tagType));
                aBitRate.setText(String.format("%d Kb/s - %d Hz - %s - %s %s", aAudioTrack.aBitRate, aAudioTrack.aFreq, aAudioTrack.aMode, aAudioTrack.aVersion, aAudioTrack.aVbr ? "VBR" : "CBR"));
                aGenre.getModel().setSelectedItem(aAudioTrack.aGenre);
                aGenre2.setSelectedItem(aAudioTrack.aGenre2);
                aYear.setText(aAudioTrack.aYear);
                aTrack.setText(aAudioTrack.aTrack);
                aComment.setText(aAudioTrack.aComment);
                aCopyright.setText(aAudioTrack.aCopyrightInfo);
                aComposer.setText(aAudioTrack.aComposer);
                aEncodedBy.setText(aAudioTrack.aEncodedBy);
                aUpdateButton.setEnabled(true);

                if (track.aPictures != null && track.aPictures.length > 0){
                    aDeletePictureButton.setEnabled(true);
                }
                else {
                    aDeletePictureButton.setEnabled(false);
                }

                aOpenFileButton.setEnabled(true);

                if (aAudioTrack.aPictures != null && aAudioTrack.aPictures.length > 0){
                    aPictureDescription.setText(aAudioTrack.aPictures[0].getDescription());
                    ImageIcon icon = new ImageIcon(aAudioTrack.aPictures[0].getPictureData());
                    icon = new ImageIcon(icon.getImage().getScaledInstance(110,110,Image.SCALE_SMOOTH));
                    aPictureModel.getPictureLabel().setIcon(icon);
                }
                else{
                    aPictureDescription.setText("");
                    aPictureModel.getPictureLabel().setIcon(JTagger.NO_IMAGE);
                    aPictureModel.setPictureData(null);
                }
            }
            else {
                clear();
                aUpdateButton.setEnabled(false);
                aDeletePictureButton.setEnabled(false);
                aOpenFileButton.setEnabled(false);
            }
        }
        catch (Exception e) {
            aUpdateButton.setEnabled(false);
            aDeletePictureButton.setEnabled(false);
            aOpenFileButton.setEnabled(false);
            JTagger.get().setMessage(e.getMessage(), true);
        }
    }


    /**
     * Write new tags to audio file.
     */
    private void writeTags() {
        try {
            aAudioTrack.aArtist = aArtist.getText();
            aAudioTrack.aAlbum = aAlbum.getText();
            aAudioTrack.aTitle = aTitle.getText();
            aAudioTrack.aYear = aYear.getText();
            aAudioTrack.aGenre = ((String) aGenre.getModel().getSelectedItem()).length() > 0 ? (String) aGenre.getModel().getSelectedItem() : "Other";
            aAudioTrack.aGenre2 = (String) aGenre2.getSelectedItem();

            if (aTrack.getText().length() == 0) {
                aAudioTrack.aTrack = "0";
            }
            else {
                aAudioTrack.aTrack = Integer.toString(Util.getNumber(aTrack.getText(), 1));
            }

            aAudioTrack.aComment = aComment.getText();
            aAudioTrack.aCopyrightInfo = aCopyright.getText();
            aAudioTrack.aEncodedBy = aEncodedBy.getText();
            aAudioTrack.aComposer = aComposer.getText();

            if (aImageSet) {
                aAudioTrack.setImage(aPictureModel, aPictureDescription.getText());
            }

            aAudioTrack.write(AudioTrack.getWriteTrackType(aAudioTrack.aFileName));
            JTagger.get().setMessage(Constants.MESSAGE_WRITE_OK, false);
            Log.get().addTime(2, String.format("Changing tags for %s", aAudioTrack.aFileName));
        }
        catch (Exception e) {
            JTagger.get().setMessage(e.getMessage(), true);
            Log.get().addTime(1, e.getMessage());

            try {
                aAudioTrack.read();
            }
            catch (Exception e1) {
            }

            setTrack(aAudioTrack);
        }

        JTagger.get().reloadAllTables();
    }
}
